@@ -958,7 +958,7 @@ ei_device_keyboard_get_keymap(struct ei_device *device)
 }
 
 static struct ei_keymap *
-ei_keymap_new(enum ei_keymap_type type, int fd, size_t size)
+ei_keymap_new(struct ei *ei, enum ei_keymap_type type, int fd, size_t size)
 {
 	_unref_(ei_keymap) *keymap = ei_keymap_create(NULL);
 
@@ -966,15 +966,20 @@ ei_keymap_new(enum ei_keymap_type type, int fd, size_t size)
 	case EI_KEYMAP_TYPE_XKB:
 		break;
 	default:
+		log_bug(ei, "Unsupported keymap type: %u", type);
 		return NULL;
 	}
 
-	if (fd < 0 || size == 0)
+	if (fd < 0 || size == 0) {
+		log_bug(ei, "Invalid keymap fd %d with size %zu", fd, size);
 		return NULL;
+	}
 
 	int newfd = xdup(fd);
-	if (newfd < 0)
+	if (newfd < 0) {
+		log_bug(ei, "Failed to dup keymap fd: %s", strerror(errno));
 		return NULL;
+	}
 
 	keymap->fd = newfd;
 	keymap->type = type;
@@ -993,12 +998,10 @@ ei_device_set_keymap(struct ei_device *device,
 	if (!type)
 		return;
 
-	_unref_(ei_keymap) *keymap = ei_keymap_new(type, keymap_fd, size);
-	if (!keymap) {
-		log_bug(ei_device_get_context(device),
-			"Failed to apply server-requested keymap");
+	struct ei *ei = ei_device_get_context(device);
+	_unref_(ei_keymap) *keymap = ei_keymap_new(ei, type, keymap_fd, size);
+	if (!keymap)
 		return; /* FIXME: ei_device_remove() here */
-	}
 
 	keymap->device = device;
 	device->keymap = ei_keymap_ref(keymap);
@@ -1362,7 +1365,7 @@ ei_device_pointer_motion(struct ei_device *device,
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1381,7 +1384,7 @@ ei_device_pointer_motion_absolute(struct ei_device *device,
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1403,7 +1406,7 @@ ei_device_button_button(struct ei_device *device,
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1428,7 +1431,7 @@ ei_device_scroll_delta(struct ei_device *device, double x, double y)
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1448,7 +1451,7 @@ ei_device_scroll_stop(struct ei_device *device, bool x, bool y)
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1477,7 +1480,7 @@ ei_device_scroll_cancel(struct ei_device *device, bool x, bool y)
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1510,7 +1513,7 @@ ei_device_scroll_discrete(struct ei_device *device, int32_t x, int32_t y)
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1553,7 +1556,7 @@ ei_device_keyboard_key(struct ei_device *device,
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1660,7 +1663,7 @@ ei_touch_down(struct ei_touch *touch, double x, double y)
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1692,7 +1695,7 @@ ei_touch_motion(struct ei_touch *touch, double x, double y)
 
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
@@ -1721,7 +1724,7 @@ ei_touch_up(struct ei_touch *touch)
 	struct ei_device *device = ei_touch_get_device(touch);
 	if (device->state != EI_DEVICE_STATE_EMULATING) {
 		log_bug_client(ei_device_get_context(device),
-			       "%s: device is not not emulating", __func__);
+			       "%s: device is not emulating", __func__);
 		return;
 	}
 
