@@ -772,7 +772,7 @@ _public_ void
 eis_device_configure_size(struct eis_device *device, uint32_t width, uint32_t height)
 {
 	if (device->type != EIS_DEVICE_TYPE_PHYSICAL) {
-		log_bug_client(eis_device_get_context(device), "Device type physical requird for size");
+		log_bug_client(eis_device_get_context(device), "Device type physical required for size");
 		return;
 	}
 
