@@ -439,6 +439,24 @@ eis_log_set_priority(struct eis *eis, enum eis_log_priority priority);
 enum eis_log_priority
 eis_log_get_priority(const struct eis *eis);
 
+/**
+ * Optional override function for eis_now().
+ *
+ * By default eis_now() returns the current timestamp in CLOCK_MONOTONIC. This
+ * may be overridden by a caller to provide a different timestamp.
+ *
+ * There is rarely a need to override this function. It exists for the libeis-internal test suite.
+ */
+typedef uint64_t (*eis_clock_now_func)(struct eis *eis);
+
+/**
+ * Override the function that returns the current time eis_now().
+ *
+ * There is rarely a need to override this function. It exists for the libeis-internal test suite.
+ */
+void
+eis_clock_set_now_func(struct eis *, eis_clock_now_func func);
+
 struct eis *
 eis_ref(struct eis *eis);
 
@@ -1594,11 +1612,11 @@ eis_event_touch_get_y(struct eis_event *event);
  * @returns a timestamp for the current time to pass into
  * eis_device_frame().
  *
- * In the current implementation, the returned timestamp is CLOCK_MONOTONIC
- * for compatibility with evdev and libinput.
+ * By default, the returned timestamp is CLOCK_MONOTONIC for compatibility with
+ * evdev and libinput. This can be overridden with eis_clock_set_now_func().
  */
 uint64_t
-eis_now(struct eis *ei);
+eis_now(struct eis *eis);
 
 /**
  * @}
