@@ -560,7 +560,7 @@ portal_select_devices(struct oeffis *oeffis)
 					&response,
 					"oa{sv}",
 					oeffis->session_path,
-					2,
+					oeffis->devices == OEFFIS_DEVICE_ALL_DEVICES ? 1 : 2,
 					"handle_token", /* string key */
 					"s", token, /* variant string */
 					"types", /* string key */
