@@ -27,6 +27,8 @@
 #include "util-macros.h"
 #include "util-io.h"
 
+_Static_assert(EAGAIN == EWOULDBLOCK, "Currently not supported, please file a bug");
+
 int
 xread_with_fds(int fd, void *buf, size_t count, int **fds)
 {
@@ -47,9 +49,7 @@ xread_with_fds(int fd, void *buf, size_t count, int **fds)
 		.msg_controllen = sizeof(control),
 	};
 
-	int received = 0;
-	with_signals_blocked(SIGALRM)
-		received = xerrno(recvmsg(fd, &msg, 0));
+	int received = xerrno(SYSCALL(recvmsg(fd, &msg, 0)));
 
 	if (received > 0) {
 		*fds = NULL;
@@ -113,10 +113,7 @@ xsend_with_fd(int fd, const void *buf, size_t len, int *fds)
 	header->cmsg_type = SCM_RIGHTS;
 	memcpy(CMSG_DATA(CMSG_FIRSTHDR(&msg)), fds, nfds * sizeof(int));
 
-	with_signals_blocked(SIGALRM)
-		return xerrno(sendmsg(fd, &msg, MSG_NOSIGNAL));
-
-	abort(); // can't be reached
+	return xerrno(SYSCALL(sendmsg(fd, &msg, MSG_NOSIGNAL)));
 }
 
 /* consider this struct opaque */
