@@ -25,9 +25,11 @@
 #include "config.h"
 
 #include <math.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/socket.h>
+#include <sys/time.h>
 
 #include <munit.h>
 
@@ -35,13 +37,17 @@
 
 #include "util-bits.h"
 #include "util-color.h"
+#include "util-io.h"
 #include "util-mem.h"
+#include "util-munit.h"
 #include "util-tristate.h"
 #include "util-logger.h"
 #include "util-object.h"
 #include "util-strings.h"
 #include "util-time.h"
 
+static bool enable_sigalarm;
+
 DEFINE_TRISTATE(yes, no, unset);
 
 struct peck {
@@ -75,6 +81,14 @@ struct peck {
 	struct eis_client *eis_client;
 
 	uint32_t indent;
+
+	struct sigaction sigact;
+
+	bool ei_fatal_bugs;
+	bool eis_fatal_bugs;
+
+	uint64_t ei_time_offset;
+	uint64_t eis_time_offset;
 };
 
 static const uint32_t INDENTATION = 4;
@@ -94,6 +108,40 @@ peck_dedent(struct peck *peck)
 static
 OBJECT_IMPLEMENT_GETTER(peck, indent, uint32_t);
 
+static uint64_t
+peck_ei_now_func(struct ei *ei)
+{
+	static uint64_t offset = 0;
+	struct peck *peck = ei_get_user_data(ei);
+
+	if (peck->ei_time_offset != offset) {
+		offset = peck->ei_time_offset;
+		log_debug(peck, "Time is now offset by %" PRIu64"us\n", offset);
+	}
+
+	uint64_t ts = 0;
+	now(&ts);
+
+	return ts + peck->ei_time_offset;
+}
+
+static uint64_t
+peck_eis_now_func(struct eis *eis)
+{
+	static uint64_t offset = 0;
+	struct peck *peck = eis_get_user_data(eis);
+
+	if (peck->eis_time_offset != offset) {
+		offset = peck->eis_time_offset;
+		log_debug(peck, "Time is now offset by %" PRIu64"us\n", offset);
+	}
+
+	uint64_t ts = 0;
+	now(&ts);
+
+	return ts + peck->ei_time_offset;
+}
+
 static void
 peck_destroy(struct peck *peck)
 {
@@ -128,6 +176,26 @@ peck_destroy(struct peck *peck)
 	ei_unref(peck->ei);
 	eis_unref(peck->eis);
 	logger_unref(peck->logger);
+
+	if (enable_sigalarm) {
+		struct itimerval timer = {0};
+		setitimer(ITIMER_REAL, &timer, 0);
+		sigaction(SIGALRM, &peck->sigact, NULL);
+	}
+}
+
+void
+peck_ei_add_time_offset(struct peck *peck, uint64_t us)
+{
+	log_debug(peck, "Adding ei time offset of %" PRIu64 "us\n", us);
+	peck->ei_time_offset += us;
+}
+
+void
+peck_eis_add_time_offset(struct peck *peck, uint64_t us)
+{
+	log_debug(peck, "Adding EIS time offset of %" PRIu64 "us\n", us);
+	peck->eis_time_offset += us;
 }
 
 static
@@ -253,32 +321,44 @@ peck_ei_get_default_touch(struct peck *peck)
 };
 
 /* Ensures that device frames in tests always have an ascending and fixed
- * interval. We start with 30 * interval in the past, every event then goes
- * forward by 10ms.
- *
- * Tests that have more than 30 events will run into the future with their
- * timestamps, shouldn't be an issue for the test suite though.
+ * interval. Every time this is called it adds 10ms to the time offset.
  */
 uint64_t
 peck_ei_now(struct peck *peck)
 {
-	const uint32_t interval = ms2us(10);
-	const uint32_t past_offset = interval * 30;
-
-	peck->now = peck->now == 0 ? ei_now(peck->ei) - past_offset : peck->now + interval;
-
-	return peck->now;
+	peck_ei_add_time_offset(peck, ms2us(10));
+	return ei_now(peck->ei);
 }
 
 uint64_t
 peck_eis_now(struct peck *peck)
 {
-	const uint32_t interval = ms2us(10);
-	const uint32_t past_offset = interval * 30;
+	peck_eis_add_time_offset(peck, ms2us(10));
+	return eis_now(peck->eis);
+}
 
-	peck->now = peck->now == 0 ? eis_now(peck->eis) - past_offset : peck->now + interval;
+void
+peck_ei_disable_fatal_bug(struct peck *peck)
+{
+	peck->ei_fatal_bugs = false;
+}
 
-	return peck->now;
+void
+peck_ei_enable_fatal_bug(struct peck *peck)
+{
+	peck->ei_fatal_bugs = true;
+}
+
+void
+peck_eis_disable_fatal_bug(struct peck *peck)
+{
+	peck->eis_fatal_bugs = false;
+}
+
+void
+peck_eis_enable_fatal_bug(struct peck *peck)
+{
+	peck->eis_fatal_bugs = true;
 }
 
 static void
@@ -294,10 +374,10 @@ peck_ei_log_handler(struct ei *ei,
 
 	const char *prefix = NULL;
 	switch (priority) {
-		case EI_LOG_PRIORITY_ERROR:   prefix = "ERR"; break;
-		case EI_LOG_PRIORITY_WARNING: prefix = "WRN"; break;
-		case EI_LOG_PRIORITY_INFO:    prefix = "INF"; break;
-		case EI_LOG_PRIORITY_DEBUG:   prefix = "   "; break;
+		case EI_LOG_PRIORITY_ERROR:   prefix = "ERR "; break;
+		case EI_LOG_PRIORITY_WARNING: prefix = "WRN "; break;
+		case EI_LOG_PRIORITY_INFO:    prefix = "INF "; break;
+		case EI_LOG_PRIORITY_DEBUG:   prefix = "    "; break;
 	}
 
 	struct peck *peck = ei_get_user_data(ei);
@@ -306,6 +386,11 @@ peck_ei_log_handler(struct ei *ei,
 		use_color ? ANSI_FG_RGB(150, 125, 100) : "",
 		message,
 		use_color ? ansi_colorcode[RESET] : "");
+
+	if (peck->ei_fatal_bugs) {
+		assert(!strstr(message, "🪳"));
+		assert(!strstr(message, "🪲"));
+	}
 }
 
 static void
@@ -321,10 +406,10 @@ peck_eis_log_handler(struct eis *eis,
 
 	const char *prefix = NULL;
 	switch (priority) {
-		case EIS_LOG_PRIORITY_ERROR:   prefix = "ERR"; break;
-		case EIS_LOG_PRIORITY_WARNING: prefix = "WRN"; break;
-		case EIS_LOG_PRIORITY_INFO:    prefix = "INF"; break;
-		case EIS_LOG_PRIORITY_DEBUG:   prefix = "   "; break;
+		case EIS_LOG_PRIORITY_ERROR:   prefix = "ERR "; break;
+		case EIS_LOG_PRIORITY_WARNING: prefix = "WRN "; break;
+		case EIS_LOG_PRIORITY_INFO:    prefix = "INF "; break;
+		case EIS_LOG_PRIORITY_DEBUG:   prefix = "    "; break;
 	}
 
 	struct peck *peck = eis_get_user_data(eis);
@@ -333,6 +418,11 @@ peck_eis_log_handler(struct eis *eis,
 		use_color ? ANSI_FG_RGB(150, 200, 125) : "",
 		message,
 		use_color ? ansi_colorcode[RESET] : "");
+
+	if (peck->eis_fatal_bugs) {
+		assert(!strstr(message, "🪳"));
+		assert(!strstr(message, "🪲"));
+	}
 }
 
 _printf_(7, 0)
@@ -350,7 +440,7 @@ peck_log_handler(struct logger *logger,
 	}
 
 	switch(priority) {
-	case LOGGER_DEBUG: msgtype = xaprintf("%3d", lineno);  break;
+	case LOGGER_DEBUG: msgtype = xaprintf("%4d", lineno);  break;
 	case LOGGER_INFO:  msgtype = xstrdup("INF");   break;
 	case LOGGER_WARN:  msgtype = xstrdup("WRN");   break;
 	case LOGGER_ERROR: msgtype = xstrdup("ERR");  break;
@@ -370,6 +460,12 @@ peck_log_handler(struct logger *logger,
 		fprintf(stderr, "%s", ansi_colorcode[RESET]);
 }
 
+static void
+handle_sigalrm(int signo)
+{
+	/* Nothing to do but this may cause a few EINTR */
+}
+
 static struct peck *
 new_context(enum peck_ei_mode ei_mode)
 {
@@ -382,6 +478,7 @@ new_context(enum peck_ei_mode ei_mode)
 	struct eis *eis = eis_new(peck);
 	eis_log_set_handler(eis, peck_eis_log_handler);
 	eis_log_set_priority(eis, EIS_LOG_PRIORITY_DEBUG);
+	eis_clock_set_now_func(eis, peck_eis_now_func);
 	rc = eis_setup_backend_fd(eis);
 	munit_assert_int(rc, ==, 0);
 	fd = eis_backend_fd_add_client(eis);
@@ -394,6 +491,7 @@ new_context(enum peck_ei_mode ei_mode)
 	ei_set_user_data(ei, peck);
 	ei_log_set_handler(ei, peck_ei_log_handler);
 	ei_log_set_priority(ei, EI_LOG_PRIORITY_DEBUG);
+	ei_clock_set_now_func(ei, peck_ei_now_func);
 	ei_configure_name(ei, "eierpecken test context");
 	rc = ei_setup_backend_fd(ei, fd);
 	munit_assert_int(rc, ==, 0);
@@ -404,10 +502,43 @@ new_context(enum peck_ei_mode ei_mode)
 	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSTART);
 	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_FRAME);
 
+	peck_ei_enable_fatal_bug(peck);
+	peck_eis_enable_fatal_bug(peck);
+
 	peck->logger = logger_new("peck", peck);
 	logger_set_handler(peck->logger, peck_log_handler);
 	logger_set_priority(peck->logger, LOGGER_DEBUG);
 
+	if (enable_sigalarm) {
+		/* We set up SIGALRM to hammer us because libei is used by Xwayland which
+		 * uses that for scheduling and we need to be able to handle that */
+		struct sigaction act;
+		sigaddset(&act.sa_mask, SIGALRM);
+		act.sa_flags = 0;
+		act.sa_handler = handle_sigalrm;
+		rc = xerrno(sigaction(SIGALRM, &act, &peck->sigact));
+		if (rc >= 0) {
+			struct itimerval timer = {
+				.it_interval = {
+					.tv_sec = 0,
+					.tv_usec = 5,
+				},
+				.it_value = {
+					.tv_sec = 0,
+					.tv_usec = 5,
+				}
+			};
+			rc = xerrno(setitimer(ITIMER_REAL, &timer, 0));
+			if (rc < 0) {
+				log_error(peck, "Failed to enable timer: %s\n", strerror(-rc));
+				munit_assert_int(rc, >=, 0);
+			}
+		} else {
+			log_error(peck, "Failed to enable SIGALRM: %s\n", strerror(-rc));
+			munit_assert_int(rc, >=, 0);
+		}
+	}
+
 	return peck;
 }
 
@@ -1035,12 +1166,12 @@ _peck_dispatch_ei(struct peck *peck, int lineno)
 }
 
 void
-_peck_dispatch_until_stable(struct peck *peck, int lineno)
+_peck_dispatch_until_stable(struct peck *peck, const char *func, int lineno)
 {
 	int eis = 0, ei = 0;
 	int loop_counter = 0;
 
-	log_debug(peck, "dispatching until stable (line %d) {\n", lineno);
+	log_debug(peck, "dispatching until stable (%s:%d) {\n", func, lineno);
 	peck_indent(peck);
 
 	/* we go two dispatch loops for both ei and EIS before we say it's
@@ -1058,7 +1189,7 @@ _peck_dispatch_until_stable(struct peck *peck, int lineno)
 	} while (ei <= 2 || eis <= 2);
 
 	peck_dedent(peck);
-	log_debug(peck, "}\n", lineno);
+	log_debug(peck, "} stable (%s:%d)\n", func, lineno);
 }
 
 void
@@ -1307,3 +1438,29 @@ _peck_mark(struct peck *peck, const char *func, int line)
 	static uint32_t mark = 0;
 	log_debug(peck, "mark %3d: line %d in %s()\n", mark++, line, func);
 }
+
+void _peck_debug(struct peck *peck, const char *func, int line, const char *format, ...)
+{
+	va_list args;
+
+	va_start(args, format);
+	log_msg_va(peck->logger, LOGGER_DEBUG, "unused", line, func, format, args);
+	va_end(args);
+}
+
+MUNIT_GLOBAL_SETUP(init_sigalarm)
+{
+	int argc = setup->argc;
+	char **argv = setup->argv;
+
+	for (int i = 0; i < argc; i++) {
+		if (streq(argv[i], "--enable-sigalrm") || streq(argv[i], "--enable-sigalarm")) {
+			enable_sigalarm = true;
+			for (int j = i + 1; j < setup->argc; j++) {
+				setup->argv[i] = setup->argv[j];
+		        }
+			setup->argc--;
+			break;
+		}
+	}
+}
