@@ -123,7 +123,8 @@ MUNIT_TEST(test_passive_ei_device_type)
 		_unref_(eis_device) *virt = eis_seat_new_device(seat);
 		eis_device_configure_type(virt, EIS_DEVICE_TYPE_VIRTUAL);
 		eis_device_configure_capability(virt, EIS_DEVICE_CAP_POINTER);
-		eis_device_configure_size(virt, 200, 200);  /* Has no effect on a virtual device */
+		with_nonfatal_eis_bug(peck)
+			eis_device_configure_size(virt, 200, 200);  /* Has no effect on a virtual device */
 		munit_assert_int(eis_device_get_type(virt), ==, EIS_DEVICE_TYPE_VIRTUAL);
 		eis_device_add(virt);
 
@@ -638,7 +639,10 @@ MUNIT_TEST(test_ei_device_regions_late_add)
 		/* region added after device_add -> ignored */
 		_unref_(eis_region) *r1 = eis_device_new_region(device);
 		eis_region_set_size(r1, 100, 200);
-		eis_region_add(r1);
+
+		/* Bug: eis_region_add: device already (dis)connected */
+		with_nonfatal_eis_bug(peck)
+			eis_region_add(r1);
 	}
 
 	peck_dispatch_until_stable(peck);
@@ -707,7 +711,8 @@ MUNIT_TEST(test_ei_device_pointer_abs)
 	with_client(peck) {
 		ei_device_close(device);
 		/* absmotion after remove must not trigger an event */
-		ei_device_pointer_motion_absolute(device, 100, 200);
+		with_nonfatal_ei_bug(peck)
+			ei_device_pointer_motion_absolute(device, 100, 200);
 	}
 
 	peck_dispatch_until_stable(peck);
@@ -995,19 +1000,32 @@ MUNIT_TEST(test_ei_device_touch)
 	with_client(peck) {
 		_unref_(ei_touch) *t = ei_device_touch_new(device);
 		/* outside clip range, expect touch to be dropped */
-		ei_touch_down(t, maxx + 1, maxy/2);
-		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_motion(t, maxx + 1, maxy/3);
-		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_up(t);
+		with_nonfatal_ei_bug(peck) {
+			ei_touch_down(t, maxx + 1, maxy/2);
+			ei_device_frame(device, peck_ei_now(peck));
+			/* ignored because the touch down was out of range */
+			ei_touch_motion(t, maxx + 1, maxy/3);
+			ei_device_frame(device, peck_ei_now(peck));
+			/* ignored because the touch down was out of range */
+			ei_touch_up(t);
+		}
 		ei_device_frame(device, peck_ei_now(peck));
+	}
 
+	peck_dispatch_until_stable(peck);
+
+	with_client(peck) {
+		_unref_(ei_touch) *t = ei_device_touch_new(device);
 		/* outside clip range, expect touch to be dropped */
-		ei_touch_down(t, maxx/2, maxy + 1);
-		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_motion(t, maxx/3, maxy + 1);
-		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_up(t);
+		with_nonfatal_ei_bug(peck) {
+			ei_touch_down(t, maxx/2, maxy + 1);
+			ei_device_frame(device, peck_ei_now(peck));
+			/* ignored because the touch down was out of range */
+			ei_touch_motion(t, maxx/3, maxy + 1);
+			ei_device_frame(device, peck_ei_now(peck));
+			/* ignored because the touch down was out of range */
+			ei_touch_up(t);
+		}
 		ei_device_frame(device, peck_ei_now(peck));
 	}
 
@@ -1022,9 +1040,12 @@ MUNIT_TEST(test_ei_device_touch)
 		ei_touch_down(t, 100, 200);
 		ei_device_frame(device, peck_ei_now(peck));
 		/* outside allowed range, generates a touch up */
-		ei_touch_motion(t, maxx + 1, 200);
+		with_nonfatal_ei_bug(peck)
+			ei_touch_motion(t, maxx + 1, 200);
 		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_up(t);
+		/* touch is already considered up */
+		with_nonfatal_ei_bug(peck)
+			ei_touch_up(t);
 		ei_device_frame(device, peck_ei_now(peck));
 	}
 
@@ -1057,14 +1078,17 @@ MUNIT_TEST(test_ei_device_touch)
 
 		/* touch never set down */
 		_unref_(ei_touch) *t2 = ei_device_touch_new(device);
-		ei_touch_up(t2);
+		with_nonfatal_ei_bug(peck)
+			ei_touch_up(t2);
 		ei_device_frame(device, peck_ei_now(peck));
 
 		/* touch never set down */
 		_unref_(ei_touch) *t3 = ei_device_touch_new(device);
-		ei_touch_motion(t3, 100, 200);
+		with_nonfatal_ei_bug(peck)
+			ei_touch_motion(t3, 100, 200);
 		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_up(t3);
+		with_nonfatal_ei_bug(peck)
+			ei_touch_up(t3);
 		ei_device_frame(device, peck_ei_now(peck));
 	}
 
@@ -1080,11 +1104,14 @@ MUNIT_TEST(test_ei_device_touch)
 		ei_device_frame(device, peck_ei_now(peck));
 		ei_touch_up(t);
 		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_down(t, 200, 300);
+		with_nonfatal_ei_bug(peck)
+			ei_touch_down(t, 200, 300);
 		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_motion(t, 300, 400);
+		with_nonfatal_ei_bug(peck)
+			ei_touch_motion(t, 300, 400);
 		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_up(t);
+		with_nonfatal_ei_bug(peck)
+			ei_touch_up(t);
 		ei_device_frame(device, peck_ei_now(peck));
 	}
 
@@ -1101,13 +1128,15 @@ MUNIT_TEST(test_ei_device_touch)
 		_unref_(ei_touch) *t = ei_device_touch_new(device);
 		ei_touch_down(t, 100, 200);
 		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_down(t, 200, 300); /* ignored */
+		with_nonfatal_ei_bug(peck)
+			ei_touch_down(t, 200, 300); /* ignored */
 		ei_device_frame(device, peck_ei_now(peck));
 		ei_touch_motion(t, 300, 400);
 		ei_device_frame(device, peck_ei_now(peck));
 		ei_touch_up(t);
 		ei_device_frame(device, peck_ei_now(peck));
-		ei_touch_up(t); /* ignored */
+		with_nonfatal_ei_bug(peck)
+			ei_touch_up(t); /* ignored */
 		ei_device_frame(device, peck_ei_now(peck));
 	}
 
@@ -1236,13 +1265,12 @@ MUNIT_TEST(test_ei_keymap_set)
 		eis_keymap_add(keymap);
 		munit_assert_ptr_equal(eis_device_keyboard_get_keymap(device), keymap);
 
-		eis_device_add(device);
-
 		/* Not possible to overwrite a keymap on a device once it's set */
 		_unref_(memfile) *fd2 = memfile_new(data, sizeof(data));
 		_unref_(eis_keymap) *overwrite =
 			eis_device_new_keymap(device, EIS_KEYMAP_TYPE_XKB, memfile_get_fd(fd2), memfile_get_size(fd2));
-		eis_keymap_add(overwrite);
+		with_nonfatal_eis_bug(peck)
+			eis_keymap_add(overwrite);
 		munit_assert_ptr_equal(eis_device_keyboard_get_keymap(device), keymap);
 
 		eis_device_add(device);
@@ -1251,7 +1279,8 @@ MUNIT_TEST(test_ei_keymap_set)
 		_unref_(eis_keymap) *ignored =
 			eis_device_new_keymap(device, EIS_KEYMAP_TYPE_XKB,
 				       memfile_get_fd(fd2), memfile_get_size(fd2));
-		eis_keymap_add(ignored);
+		with_nonfatal_eis_bug(peck)
+			eis_keymap_add(ignored);
 		munit_assert_ptr_equal(eis_device_keyboard_get_keymap(device), keymap);
 	}
 
@@ -1474,7 +1503,8 @@ MUNIT_TEST(test_ei_flush_frame)
 		struct ei_device *device = peck_ei_get_default_keyboard(peck);
 		ei_device_keyboard_key(device, KEY_Q, true);
 		/* Missing call to ei_device_frame() */
-		ei_device_stop_emulating(device);
+		with_nonfatal_ei_bug(peck)
+			ei_device_stop_emulating(device);
 	}
 
 	peck_dispatch_until_stable(peck);
@@ -2219,19 +2249,32 @@ MUNIT_TEST(test_passive_ei_device_touch)
 	with_server(peck) {
 		_unref_(eis_touch) *t = eis_device_touch_new(device);
 		/* outside clip range, expect touch to be dropped */
-		eis_touch_down(t, maxx + 1, maxy/2);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_down(t, maxx + 1, maxy/2);
 		eis_device_frame(device, peck_eis_now(peck));
+		/* ignored because the touch down was out of range */
 		eis_touch_motion(t, maxx + 1, maxy/3);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_up(t);
+		/* ignored because the touch down was out of range */
+		with_nonfatal_eis_bug(peck)
+			eis_touch_up(t);
 		eis_device_frame(device, peck_eis_now(peck));
+	}
 
+	peck_dispatch_until_stable(peck);
+
+	with_server(peck) {
+		_unref_(eis_touch) *t = eis_device_touch_new(device);
 		/* outside clip range, expect touch to be dropped */
-		eis_touch_down(t, maxx/2, maxy + 1);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_down(t, maxx/2, maxy + 1);
 		eis_device_frame(device, peck_eis_now(peck));
+		/* ignored because the touch down was out of range */
 		eis_touch_motion(t, maxx/3, maxy + 1);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_up(t);
+		/* ignored because the touch down was out of range */
+		with_nonfatal_eis_bug(peck)
+			eis_touch_up(t);
 		eis_device_frame(device, peck_eis_now(peck));
 	}
 
@@ -2246,9 +2289,12 @@ MUNIT_TEST(test_passive_ei_device_touch)
 		eis_touch_down(t, 100, 200);
 		eis_device_frame(device, peck_eis_now(peck));
 		/* outside allowed range, generates a touch up */
-		eis_touch_motion(t, maxx + 1, 200);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_motion(t, maxx + 1, 200);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_up(t);
+		/* touch is already considered up */
+		with_nonfatal_eis_bug(peck)
+			eis_touch_up(t);
 		eis_device_frame(device, peck_eis_now(peck));
 	}
 
@@ -2277,19 +2323,25 @@ MUNIT_TEST(test_passive_ei_device_touch)
 
 	with_server(peck) {
 		/* touch only allocated, not actually set down */
-		_unref_(eis_touch) *t1 = eis_device_touch_new(device);
+		struct eis_touch *t1 = eis_device_touch_new(device);
 
 		/* touch never set down */
 		_unref_(eis_touch) *t2 = eis_device_touch_new(device);
-		eis_touch_up(t2);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_up(t2);
 		eis_device_frame(device, peck_eis_now(peck));
 
 		/* touch never set down */
 		_unref_(eis_touch) *t3 = eis_device_touch_new(device);
-		eis_touch_motion(t3, 100, 200);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_motion(t3, 100, 200);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_up(t3);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_up(t3);
 		eis_device_frame(device, peck_eis_now(peck));
+
+		with_nonfatal_eis_bug(peck)
+			eis_touch_unref(t1);
 	}
 
 	peck_dispatch_until_stable(peck);
@@ -2304,11 +2356,14 @@ MUNIT_TEST(test_passive_ei_device_touch)
 		eis_device_frame(device, peck_eis_now(peck));
 		eis_touch_up(t);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_down(t, 200, 300);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_down(t, 200, 300);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_motion(t, 300, 400);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_motion(t, 300, 400);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_up(t);
+		with_nonfatal_eis_bug(peck)
+			eis_touch_up(t);
 		eis_device_frame(device, peck_eis_now(peck));
 	}
 
@@ -2325,13 +2380,15 @@ MUNIT_TEST(test_passive_ei_device_touch)
 		_unref_(eis_touch) *t = eis_device_touch_new(device);
 		eis_touch_down(t, 100, 200);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_down(t, 200, 300); /* ignored */
+		with_nonfatal_eis_bug(peck)
+			eis_touch_down(t, 200, 300); /* ignored */
 		eis_device_frame(device, peck_eis_now(peck));
 		eis_touch_motion(t, 300, 400);
 		eis_device_frame(device, peck_eis_now(peck));
 		eis_touch_up(t);
 		eis_device_frame(device, peck_eis_now(peck));
-		eis_touch_up(t); /* ignored */
+		with_nonfatal_eis_bug(peck)
+			eis_touch_up(t); /* ignored */
 		eis_device_frame(device, peck_eis_now(peck));
 	}
 
@@ -2496,7 +2553,8 @@ MUNIT_TEST(test_passive_ei_flush_frame)
 		eis_device_start_emulating(device, sequence);
 		eis_device_keyboard_key(device, KEY_Q, true);
 		/* Missing call to ei_device_frame() */
-		eis_device_stop_emulating(device);
+		with_nonfatal_eis_bug(peck)
+			eis_device_stop_emulating(device);
 	}
 
 	peck_dispatch_until_stable(peck);
