@@ -28,6 +28,7 @@
 #include <fcntl.h>
 
 #include "util-munit.h"
+#include "util-time.h"
 #include "util-strings.h"
 #include "eierpecken.h"
 
@@ -545,3 +546,210 @@ MUNIT_TEST(test_xdotool_rel_motion)
 
 	return MUNIT_OK;
 }
+
+MUNIT_TEST(test_ei_exceed_write_buffer)
+{
+	_unref_(peck) *peck = peck_new();
+
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_NONE);
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ACCEPT_ALL);
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_POINTER);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTODEVICES);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSTART);
+	peck_dispatch_until_stable(peck);
+
+	uint64_t toffset = peck_ei_now(peck);
+	unsigned int count = 10000; /* Large enough to require several flushes */
+	struct eis_event *events[count];
+
+	with_server(peck) {
+		peck_drain_eis(eis);
+	}
+
+	with_client(peck) {
+		struct ei_device *device = peck_ei_get_default_pointer(peck);
+		for (unsigned int i = 0; i < count/2; i++) {
+			ei_device_pointer_motion(device, -1, 10);
+			ei_device_frame(device, toffset + i);
+		}
+	}
+
+	peck_dispatch_eis(peck);
+
+	unsigned int before_buffer = 0;
+	with_server(peck) {
+		struct eis_event *next;
+		while ((next = eis_get_event(eis))) {
+			events[before_buffer++] = next;
+			munit_assert_uint(before_buffer, <=, count);
+		}
+	}
+
+	unsigned int nevents = before_buffer;
+	if (before_buffer < count) {
+		/* We sent >socket buffersize events, so we don't expect to receive all of those */
+		/* Calling dispatch (on both) should flush some more */
+		peck_dispatch_until_stable(peck);
+
+		with_server(peck) {
+			struct eis_event *next;
+			while ((next = eis_get_event(eis))) {
+				events[nevents++] = next;
+
+				munit_assert_uint(nevents, <=, count);
+
+				if (nevents % 50 == 0)
+					peck_dispatch_ei(peck);
+
+				_unref_(eis_event) *next = eis_peek_event(eis);
+				if (!next)
+					peck_dispatch_eis(peck);
+			}
+		}
+	};
+
+	munit_assert_uint(nevents, ==, count);
+
+	for (unsigned int i = 0; i < count; i += 2) {
+		_unref_(eis_event) *motion = events[i];
+		_unref_(eis_event) *frame = events[i+1];
+
+		uint64_t time = eis_event_get_time(frame);
+
+		munit_assert_string_equal(peck_eis_event_name(motion),
+					  peck_eis_event_type_name(EIS_EVENT_POINTER_MOTION));
+		munit_assert_string_equal(peck_eis_event_name(frame),
+					  peck_eis_event_type_name(EIS_EVENT_FRAME));
+		munit_assert_int64(time, ==, toffset + i/2);
+	}
+
+	/* Our events are as expected but we never got EAGAIN on
+	 * the buffer, so let's count this test as skipped */
+	if (before_buffer == count)
+		return MUNIT_SKIP;
+
+	return MUNIT_OK;
+}
+
+MUNIT_TEST(test_ei_exceed_write_buffer_cleanup)
+{
+	_unref_(peck) *peck = peck_new();
+	struct ei *ei = peck_get_ei(peck);
+
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_NONE);
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ACCEPT_ALL);
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_POINTER);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTODEVICES);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSTART);
+	peck_dispatch_until_stable(peck);
+
+	uint64_t toffset = peck_ei_now(peck);
+	unsigned int count = 10000; /* Large enough to require several flushes */
+
+	with_server(peck) {
+		peck_drain_eis(eis);
+	}
+
+	with_client(peck) {
+		struct ei_device *device = peck_ei_get_default_pointer(peck);
+		for (unsigned int i = 0; i < count/2; i++) {
+			ei_device_pointer_motion(device, -1, 10);
+			ei_device_frame(device, toffset + i);
+		}
+	}
+
+	peck_dispatch_eis(peck);
+
+	unsigned int before_buffer = 0;
+	with_server(peck) {
+		struct eis_event *next;
+		while ((next = eis_get_event(eis))) {
+			munit_assert_uint(before_buffer, <=, count);
+			eis_event_unref(next);
+		}
+	}
+
+	/* Our events are as expected but we never got EAGAIN on
+	 * the buffer, so let's count this test as skipped */
+	if (before_buffer == count)
+		return MUNIT_SKIP;
+
+	/* Make sure cleanup is handled properly */
+	peck_drop_ei(peck);
+	ei_unref(ei);
+
+	return MUNIT_OK;
+}
+
+MUNIT_TEST(test_ei_invalid_object_ids)
+{
+	_unref_(peck) *peck = peck_new();
+	_unref_(eis_device) *eis_device = NULL;
+	_unref_(eis_device) *dummy = NULL;
+
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_NONE);
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ACCEPT_ALL);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTODEVICES);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSTART);
+
+	peck_dispatch_until_stable(peck);
+
+	with_server(peck) {
+		struct eis_seat *seat = peck_eis_get_default_seat(peck);
+		eis_device = eis_seat_new_device(seat);
+		eis_device_configure_name(eis_device, __func__);
+		eis_device_configure_capability(eis_device, EIS_DEVICE_CAP_POINTER);
+		eis_device_add(eis_device);
+		eis_device_resume(eis_device);
+	}
+
+	/* Client has device now, remove it from server but don't tell client yet */
+	peck_dispatch_until_stable(peck);
+
+	with_server(peck) {
+		eis_device_pause(eis_device);
+		eis_device_remove(eis_device);
+	}
+	peck_dispatch_eis(peck);
+
+	with_client(peck) {
+		struct ei_device *device = peck_ei_get_default_pointer(peck);
+		ei_device_start_emulating(device, 0);
+		ei_device_pointer_motion(device, -1, 1);
+		ei_device_frame(device, ei_now(ei));
+		ei_device_stop_emulating(device);
+	}
+
+	/* This call should trigger invalid object messages */
+	peck_dispatch_eis(peck);
+
+	/* This call should receive the invalid object messages */
+	peck_dispatch_ei(peck);
+
+	/* Create a second device to test the defunct cleaning code, this one is
+	 * just used to send messages that will trigger ei_dispatch() */
+	with_server(peck) {
+		struct eis_seat *seat = peck_eis_get_default_seat(peck);
+		dummy = eis_seat_new_device(seat);
+		eis_device_configure_name(dummy, __func__);
+		eis_device_configure_capability(dummy, EIS_DEVICE_CAP_POINTER);
+		eis_device_add(dummy);
+		eis_device_resume(dummy);
+	}
+
+	peck_dispatch_until_stable(peck);
+
+	/* trigger the defunct object code - we can't actually test anything here
+	 * beyond hoping it crashes if there's a bug */
+	peck_ei_add_time_offset(peck, s2us(6));
+
+	for (int i = 0; i < 30; i++) {
+		with_server(peck) {
+			eis_device_pause(dummy);
+			eis_device_resume(dummy);
+		}
+		peck_dispatch_ei(peck);
+	}
+
+	return MUNIT_OK;
+}
