@@ -53,6 +53,7 @@ from dbus.mainloop.glib import DBusGMainLoop
 import attr
 import ctypes
 import dbus
+import dbus.proxies
 import dbusmock
 import fcntl
 import os
@@ -317,11 +318,12 @@ def session_bus(session_bus_unmonitored) -> Iterator[dbusmock.DBusTestCase]:
 
 
 @pytest.fixture()
-def daemon(
+def mock(
     session_bus, portal_name="RemoteDesktop", params=None, extra_templates=[]
-) -> Iterator[subprocess.Popen]:
+) -> Iterator[dbus.proxies.ProxyObject]:
     """
-    Fixture that starts a DBusMock daemon in a separate process.
+    Fixture that starts a DBusMock daemon in a separate process and returns
+    the ProxyObject for the mock.
 
     If extra_templates is specified, it is a list of tuples with the
     portal name as first value and the param dict to be passed to that
@@ -344,7 +346,7 @@ def daemon(
             dbus_interface=dbusmock.MOCK_IFACE,
         )
 
-    yield p_mock
+    yield obj_portal
 
     if p_mock.stdout:
         out = (p_mock.stdout.read() or b"").decode("utf-8")
@@ -379,7 +381,7 @@ class TestOeffis:
     Test class that sets up a mocked DBus session bus to be used by liboeffis.so.
     """
 
-    def test_create_session(self, daemon, mainloop):
+    def test_create_session(self, mock, mainloop):
         oeffis = Oeffis()
         oeffis.create_session(oeffis.DEVICE_POINTER | oeffis.DEVICE_KEYBOARD)  # type: ignore
         oeffis.dispatch()  # type: ignore
@@ -400,6 +402,30 @@ class TestOeffis:
 
         assert eisfd.recv(64) == b"VANILLA"  # that's what the template sends
 
+    def test_create_session_all_devices(self, mock, mainloop):
+        oeffis = Oeffis()
+        oeffis.create_session(oeffis.DEVICE_ALL_DEVICES)  # type: ignore
+        oeffis.dispatch()  # type: ignore
+
+        def _dispatch(source, condition):
+            oeffis.dispatch()  # type: ignore
+            return True
+
+        GLib.io_add_watch(oeffis.fd, 0, GLib.IO_IN, _dispatch)
+
+        mainloop.run()
+        e = oeffis.get_event()  # type: ignore
+        assert e == oeffis.EVENT_CONNECTED_TO_EIS, oeffis.error_message  # type: ignore
+
+        mock_interface = dbus.Interface(mock, dbusmock.MOCK_IFACE)
+        method_calls = mock_interface.GetMethodCalls("SelectDevices")
+        assert len(method_calls) > 0
+        _, args = method_calls[-1]
+        options = args[1]
+        assert "handle_token" in options
+        # if OEFFIS_DEVICE_ALL_DEVICES is selected, liboeffis skips the types option
+        assert "types" not in options
+
 
 def test_version_compare():
     assert version_at_least("1", "1.0")
