@@ -550,7 +550,7 @@ static void
 usage(FILE *fp, const char *argv0)
 {
 	fprintf(fp,
-		"Usage: %s [--verbose] [--uinput] [--socketpath=/path/to/socket]\n"
+		"Usage: %s [--verbose] [--uinput] [--socketpath=/path/to/socket] [--interval=1000]\n"
 		"\n"
 		"Start an EIS demo server. The server accepts all client connections\n"
 		"and devices and prints any events from the client to stdout.\n"
@@ -559,6 +559,7 @@ usage(FILE *fp, const char *argv0)
 		" --socketpath	Use the given socket path. Default: $XDG_RUNTIME_DIR/eis-0\n"
 		" --layout	Use the given XKB layout (requires libxkbcommon). Default: none\n"
 		" --uinput	Set up each device as uinput device (this requires root)\n"
+		" --interval    Interval in milliseconds between polling\n"
 		" --verbose	Enable debugging output\n"
 		"",
 		argv0);
@@ -568,6 +569,7 @@ int main(int argc, char **argv)
 {
 	bool verbose = false;
 	bool uinput = false;
+	unsigned int interval = 1000;
 	const char *layout = NULL;
 
 	_cleanup_unlink_free_ char *socketpath = NULL;
@@ -581,12 +583,14 @@ int main(int argc, char **argv)
 			OPT_LAYOUT,
 			OPT_SOCKETPATH,
 			OPT_UINPUT,
+			OPT_INTERVAL,
 		};
 		static struct option long_opts[] = {
 			{"socketpath",	required_argument, 0, OPT_SOCKETPATH},
 			{"layout",	required_argument, 0, OPT_LAYOUT},
 			{"uinput",	no_argument, 0, OPT_UINPUT},
 			{"verbose",	no_argument, 0, OPT_VERBOSE},
+			{"interval",    required_argument, 0, OPT_INTERVAL},
 			{"help",	no_argument, 0, 'h'},
 			{NULL},
 		};
@@ -613,6 +617,9 @@ int main(int argc, char **argv)
 			case OPT_VERBOSE:
 				verbose = true;
 				break;
+			case OPT_INTERVAL:
+				interval = atoi(optarg);
+				break;
 			default:
 				usage(stderr, argv[0]);
 				return EXIT_FAILURE;
@@ -668,7 +675,7 @@ int main(int argc, char **argv)
 	};
 
 	int nevents;
-	while (!stop && (nevents = poll(&fds, 1, 1000)) > -1) {
+	while (!stop && (nevents = poll(&fds, 1, interval)) > -1) {
 		if (nevents == 0 && server.nreceiver_clients == 0)
 			continue;
 
